"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var l10n_1 = require("./l10n");
var HOURS_IN_DAY = 8;
function formatMeasure(value, type, options) {
    var formatter = getFormatter(type);
    return useFormatter(value, formatter, options);
}
exports.formatMeasure = formatMeasure;
function localizeMetric(metricKey) {
    return l10n_1.translate('metric', metricKey, 'name');
}
exports.localizeMetric = localizeMetric;
function getShortType(type) {
    if (type === 'INT') {
        return 'SHORT_INT';
    }
    else if (type === 'WORK_DUR') {
        return 'SHORT_WORK_DUR';
    }
    return type;
}
exports.getShortType = getShortType;
function isDiffMetric(metricKey) {
    return metricKey.indexOf('new_') === 0;
}
exports.isDiffMetric = isDiffMetric;
var DEFAULT_DECIMALS = 1;
function getMinDecimalsCountToBeDistinctFromThreshold(value, threshold) {
    if (!threshold) {
        return DEFAULT_DECIMALS;
    }
    var delta = Math.abs(threshold - value);
    if (delta < 0.1 && delta > 0) {
        return -Math.floor(Math.log10(delta));
    }
    else {
        return DEFAULT_DECIMALS;
    }
}
exports.getMinDecimalsCountToBeDistinctFromThreshold = getMinDecimalsCountToBeDistinctFromThreshold;
function useFormatter(value, formatter, options) {
    return value !== undefined && value !== '' ? formatter(value, options) : '';
}
function getFormatter(type) {
    var FORMATTERS = {
        INT: intFormatter,
        SHORT_INT: shortIntFormatter,
        FLOAT: floatFormatter,
        PERCENT: percentFormatter,
        WORK_DUR: durationFormatter,
        SHORT_WORK_DUR: shortDurationFormatter,
        RATING: ratingFormatter,
        LEVEL: levelFormatter,
        MILLISEC: millisecondsFormatter,
    };
    return FORMATTERS[type] || noFormatter;
}
function numberFormatter(value, minimumFractionDigits, maximumFractionDigits) {
    if (minimumFractionDigits === void 0) { minimumFractionDigits = 0; }
    if (maximumFractionDigits === void 0) { maximumFractionDigits = minimumFractionDigits; }
    var format = new Intl.NumberFormat(l10n_1.getCurrentLocale(), {
        minimumFractionDigits: minimumFractionDigits,
        maximumFractionDigits: maximumFractionDigits,
    }).format;
    if (typeof value === 'string') {
        return format(parseFloat(value));
    }
    return format(value);
}
function noFormatter(value) {
    return value;
}
function intFormatter(value) {
    return numberFormatter(value);
}
var shortIntFormats = [
    { unit: 1e10, formatUnit: 1e9, fraction: 0, suffix: 'short_number_suffix.g' },
    { unit: 1e9, formatUnit: 1e9, fraction: 1, suffix: 'short_number_suffix.g' },
    { unit: 1e7, formatUnit: 1e6, fraction: 0, suffix: 'short_number_suffix.m' },
    { unit: 1e6, formatUnit: 1e6, fraction: 1, suffix: 'short_number_suffix.m' },
    { unit: 1e4, formatUnit: 1e3, fraction: 0, suffix: 'short_number_suffix.k' },
    { unit: 1e3, formatUnit: 1e3, fraction: 1, suffix: 'short_number_suffix.k' },
];
function shortIntFormatter(value, option) {
    var roundingFunc = (option && option.roundingFunc) || undefined;
    if (typeof value === 'string') {
        value = parseFloat(value);
    }
    for (var i = 0; i < shortIntFormats.length; i++) {
        var _a = shortIntFormats[i], unit = _a.unit, formatUnit = _a.formatUnit, fraction = _a.fraction, suffix = _a.suffix;
        var nextFraction = unit / (shortIntFormats[i + 1] ? shortIntFormats[i + 1].unit / 10 : 1);
        var roundedValue = numberRound(value / unit, nextFraction, roundingFunc);
        if (roundedValue >= 1) {
            return (numberFormatter(numberRound(value / formatUnit, Math.pow(10, fraction), roundingFunc), 0, fraction) + l10n_1.translate(suffix));
        }
    }
    return numberFormatter(value);
}
function numberRound(value, fraction, roundingFunc) {
    if (fraction === void 0) { fraction = 1000; }
    if (roundingFunc === void 0) { roundingFunc = Math.round; }
    return roundingFunc(value * fraction) / fraction;
}
function floatFormatter(value) {
    return numberFormatter(value, 1, 5);
}
function percentFormatter(value, _a) {
    var _b = _a === void 0 ? {} : _a, decimals = _b.decimals, omitExtraDecimalZeros = _b.omitExtraDecimalZeros;
    if (typeof value === 'string') {
        value = parseFloat(value);
    }
    if (value === 100) {
        return '100%';
    }
    else if (omitExtraDecimalZeros && decimals) {
        return numberFormatter(value, 1, decimals) + "%";
    }
    else {
        return numberFormatter(value, decimals || 1) + "%";
    }
}
function ratingFormatter(value) {
    if (typeof value === 'string') {
        value = parseInt(value, 10);
    }
    return String.fromCharCode(97 + value - 1).toUpperCase();
}
function levelFormatter(value) {
    if (typeof value === 'number') {
        value = value.toString();
    }
    var l10nKey = "metric.level." + value;
    var result = l10n_1.translate(l10nKey);
    return l10nKey !== result ? result : value;
}
function millisecondsFormatter(value) {
    if (typeof value === 'string') {
        value = parseInt(value, 10);
    }
    var ONE_SECOND = 1000;
    var ONE_MINUTE = 60 * ONE_SECOND;
    if (value >= ONE_MINUTE) {
        var minutes = Math.round(value / ONE_MINUTE);
        return minutes + "min";
    }
    else if (value >= ONE_SECOND) {
        var seconds = Math.round(value / ONE_SECOND);
        return seconds + "s";
    }
    else {
        return value + "ms";
    }
}
function shouldDisplayDays(days) {
    return days > 0;
}
function shouldDisplayDaysInShortFormat(days) {
    return days > 0.9;
}
function shouldDisplayHours(days, hours) {
    return hours > 0 && days < 10;
}
function shouldDisplayHoursInShortFormat(hours) {
    return hours > 0.9;
}
function shouldDisplayMinutes(days, hours, minutes) {
    return minutes > 0 && hours < 10 && days === 0;
}
function addSpaceIfNeeded(value) {
    return value.length > 0 ? value + " " : value;
}
function formatDuration(isNegative, days, hours, minutes) {
    var formatted = '';
    if (shouldDisplayDays(days)) {
        formatted += l10n_1.translateWithParameters('work_duration.x_days', isNegative ? -1 * days : days);
    }
    if (shouldDisplayHours(days, hours)) {
        formatted = addSpaceIfNeeded(formatted);
        formatted += l10n_1.translateWithParameters('work_duration.x_hours', isNegative && formatted.length === 0 ? -1 * hours : hours);
    }
    if (shouldDisplayMinutes(days, hours, minutes)) {
        formatted = addSpaceIfNeeded(formatted);
        formatted += l10n_1.translateWithParameters('work_duration.x_minutes', isNegative && formatted.length === 0 ? -1 * minutes : minutes);
    }
    return formatted;
}
function formatDurationShort(isNegative, days, hours, minutes) {
    if (shouldDisplayDaysInShortFormat(days)) {
        var roundedDays = Math.round(days);
        var formattedDays = formatMeasure(isNegative ? -1 * roundedDays : roundedDays, 'SHORT_INT');
        return l10n_1.translateWithParameters('work_duration.x_days', formattedDays);
    }
    if (shouldDisplayHoursInShortFormat(hours)) {
        var roundedHours = Math.round(hours);
        var formattedHours = formatMeasure(isNegative ? -1 * roundedHours : roundedHours, 'SHORT_INT');
        return l10n_1.translateWithParameters('work_duration.x_hours', formattedHours);
    }
    var formattedMinutes = formatMeasure(isNegative ? -1 * minutes : minutes, 'SHORT_INT');
    return l10n_1.translateWithParameters('work_duration.x_minutes', formattedMinutes);
}
function durationFormatter(value) {
    if (typeof value === 'string') {
        value = parseInt(value, 10);
    }
    if (value === 0) {
        return '0';
    }
    var hoursInDay = HOURS_IN_DAY;
    var isNegative = value < 0;
    var absValue = Math.abs(value);
    var days = Math.floor(absValue / hoursInDay / 60);
    var remainingValue = absValue - days * hoursInDay * 60;
    var hours = Math.floor(remainingValue / 60);
    remainingValue -= hours * 60;
    return formatDuration(isNegative, days, hours, remainingValue);
}
function shortDurationFormatter(value) {
    if (typeof value === 'string') {
        value = parseInt(value, 10);
    }
    if (value === 0) {
        return '0';
    }
    var hoursInDay = HOURS_IN_DAY;
    var isNegative = value < 0;
    var absValue = Math.abs(value);
    var days = absValue / hoursInDay / 60;
    var remainingValue = absValue - Math.floor(days) * hoursInDay * 60;
    var hours = remainingValue / 60;
    remainingValue -= Math.floor(hours) * 60;
    return formatDurationShort(isNegative, days, hours, remainingValue);
}
//# sourceMappingURL=measures.js.map